-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

Swords = {}
Swords.name = "Swords"
Swords.slashName = "swords"
Swords.version = "v1.5.1"
Swords.x = 500
Swords.y = 500
Swords.visible = true
Swords.mode = "combat"
Swords.image = "dagger"
Swords.imageOffset = 50
Swords.backgroundAlpha = 0
Swords.maxSwords = 3
Swords.imageScale = 1
Swords.inCombat = false

--
-- Main functions
--

function Swords.Create()
	local parent = UI.CreateContext(Swords.name .. "Context")
	Swords.parentFrame = UI.CreateFrame("Frame", Swords.name .. "Parent", parent);
	Swords.parentFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Swords.x, Swords.y)
	
	Swords.CreateImageFrames()
end

function Swords.CreateImage(slot, indx, x, y)
	local frame = UI.CreateFrame("Texture", Swords.name .. slot .. indx, Swords.parentFrame)
	frame:SetVisible(false)
	frame:SetTexture(Swords.name, Swords.image .. "_" .. indx .. ".png")
	frame:SetPoint("TOPLEFT", Swords.parentFrame, "TOPLEFT", x, y)
	if Swords.imageScale ~= 1 then
		local width = math.floor(frame:GetWidth() * Swords.imageScale + 0.5)
		local height = math.floor(frame:GetHeight() * Swords.imageScale + 0.5)
		if width > 4 and width < UIParent:GetWidth() and
		   height > 4 and height < UIParent:GetHeight() then
			frame:SetWidth(width)
			frame:SetHeight(height)
		end
	end
	
	return frame
end

function Swords.CreateImageSet(slot, x, y)
	local frame1 = Swords.CreateImage(slot, "0", x, y)
	local frame2 = Swords.CreateImage(slot, "1", x, y)
	local frame3 = Swords.CreateImage(slot, "2", x, y)
	local frame4 = Swords.CreateImage(slot, "3", x, y)
	return { frame1, frame2, frame3, frame4 }
end

function Swords.CreateImageFrames()
	local x = 0
	local y = 0
	
	Swords.frames1 = Swords.CreateImageSet("1", x, y)

	x = x + Swords.imageOffset

	Swords.frames2 = Swords.CreateImageSet("2", x, y)

	x = x + Swords.imageOffset
	
	Swords.frames3 = Swords.CreateImageSet("3", x, y)

	if Swords.maxSwords < 5 then
		local maxX = x + Swords.frames3[1]:GetWidth()
		Swords.parentFrame:SetWidth(maxX)
		Swords.parentFrame:SetHeight(Swords.frames3[1]:GetHeight())
	end
	
	x = x + Swords.imageOffset
	
	Swords.frames4 = Swords.CreateImageSet("4", x, y)
	
	x = x + Swords.imageOffset
	
	Swords.frames5 = Swords.CreateImageSet("5", x, y)
	
	if Swords.maxSwords >= 5 then
		local maxX = x + Swords.frames5[1]:GetWidth()
		Swords.parentFrame:SetWidth(maxX)
		Swords.parentFrame:SetHeight(Swords.frames5[1]:GetHeight())
	end
	
	if Swords.backgroundAlpha > 0 then
		Swords.parentFrame:SetBackgroundColor(0,0,0,Swords.backgroundAlpha)
	else
		Swords.parentFrame:SetWidth(0)
		Swords.parentFrame:SetHeight(0)
	end
	
	Swords.frame = 1
	Swords.frameIncrement = 1
end

function Swords.EnableDragFrame()
	local width = Swords.maxSwords * 50
	local height = 50
	local dragFrame = {}
	dragFrame.dragging = false
	dragFrame.frame = UI.CreateFrame("Frame", "SwordsDrag", Swords.parentFrame)
	dragFrame.frame:SetPoint("TOPLEFT", Swords.parentFrame, "TOPLEFT")
	dragFrame.frame:SetPoint("BOTTOMRIGHT", Swords.parentFrame, "TOPLEFT", width, height)
	dragFrame.width = width
	dragFrame.height = height
	dragFrame.changedCallback = changedCallback
	dragFrame.frame:SetMouseMasking("limited")
	dragFrame.frame.Event.RightUp = function () Swords.MouseUp(dragFrame) end
	dragFrame.frame.Event.RightUpoutside = function () Swords.MouseUp(dragFrame) end
	dragFrame.frame.Event.RightDown = function () Swords.MouseDown(dragFrame) end
	dragFrame.frame.Event.MouseMove = function (event, x, y) Swords.MouseMove(dragFrame, x, y) end
	Swords.dragFrame = dragFrame
end

function Swords.DisableDragFrame()
	if Swords.dragFrame then
		Swords.dragFrame.frame:SetVisible(false)
		Swords.dragFrame.frame = nil
		Swords.dragFrame = nil
	end
end

function Swords.SetVisible()
	if not Swords.frame or Swords.frame < 1 then
		return
	end
	
	if not Swords.swords then
		return
	end
	
	local visible = Swords.visible
	if visible == true and Swords.mode == "combat" and Swords.inCombat == false then
		visible = false
	end
	
	Swords.parentFrame:SetVisible(visible)
	
	if visible == true then
		Swords.frames1[Swords.frame]:SetVisible(false)
		Swords.frames2[Swords.frame]:SetVisible(false)
		Swords.frames3[Swords.frame]:SetVisible(false)
		Swords.frames4[Swords.frame]:SetVisible(false)
		Swords.frames5[Swords.frame]:SetVisible(false)
		
		if Swords.swords == Swords.maxSwords and Swords.inCombat == true then
			if not Swords.timerID then
				Swords.timerID = SwordsTimer.AddCallback("SwordsRedraw", 0.1, Swords.SetVisible, true)
			end
			
			Swords.frame = Swords.frame + Swords.frameIncrement
			if Swords.frame > #Swords.frames1 then
				Swords.frameIncrement = -1
				Swords.frame = #Swords.frames1 + Swords.frameIncrement
			elseif Swords.frame < 1 then
				Swords.frameIncrement = 1
				Swords.frame = 2
			end
		else
			if Swords.timerID then
				SwordsTimer.RemoveCallback(Swords.timerID)
				Swords.timerID = nil
			end
			
			Swords.frame = 1
			Swords.frameIncrement = 1
		end

		if Swords.swords > 0 then
			Swords.frames1[Swords.frame]:SetVisible(true)
			if Swords.swords > 1 then
				Swords.frames2[Swords.frame]:SetVisible(true)
				if Swords.swords > 2 then
					Swords.frames3[Swords.frame]:SetVisible(true)
					if Swords.swords > 3 then
						Swords.frames4[Swords.frame]:SetVisible(true)
						if Swords.swords > 4 then
							Swords.frames5[Swords.frame]:SetVisible(true)
						end
					end
				end
			end
		end
	else
		Swords.HideSwords()

		Swords.frame = 1
		Swords.frameIncrement = 1
		
		if Swords.timerID then
			SwordsTimer.RemoveCallback(Swords.timerID)
			Swords.timerID = nil
		end
	end
end

function Swords.HideSwords()
	for i=1,#Swords.frames1 do
		Swords.frames1[i]:SetVisible(false)
		Swords.frames2[i]:SetVisible(false)
		Swords.frames3[i]:SetVisible(false)
		Swords.frames4[i]:SetVisible(false)
		Swords.frames5[i]:SetVisible(false)
	end
end


function Swords.GetPlayerID()
	if not Swords.playerID then
		Swords.playerID = Inspect.Unit.Lookup("player")
	end
	
	return Swords.playerID
end

function Swords.Start()
	local playerID = Swords.GetPlayerID()
	if not playerID then
		print("Swords couldn't find the player ID")
		return
	end

	Swords.swords = 0
	local detail = Inspect.Unit.Detail(playerID)
	if detail then
		if detail.combo then
			Swords.swords = detail.combo
		end
		
		if detail.calling then
			if detail.calling == "warrior" then
				Swords.maxSwords = 3
			elseif detail.calling == "rogue" then
				Swords.maxSwords = 5
			else
				Swords.visible = false
			end
		end
	end
	
	Swords.Create()	
	Swords.SetVisible()
end

function Swords.SetImage(imageName)
	if not imageName then
		Swords.image = "dagger"
	else
		Swords.image = imageName
	end
	
	Swords.HideSwords()
	Swords.CreateImageFrames()
	Swords.SetVisible()
end

function Swords.SetImageOffset(imageOffset)
	if not tonumber(imageOffset) then
		Swords.imageOffset = 50
	else
		Swords.imageOffset = tonumber(imageOffset)
	end
	
	Swords.HideSwords()
	Swords.CreateImageFrames()
	Swords.SetVisible()
end

function Swords.SetImageScale(imageScale)
	if not tonumber(imageScale) then
		Swords.imageScale = 1
	else
		Swords.imageScale = tonumber(imageScale)
	end
	
	if Swords.imageScale < 0.001 then
		Swords.imageScale = 1
	end
	
	Swords.HideSwords()
	Swords.CreateImageFrames()
	Swords.SetVisible()
end

function Swords.SetBackgroundAlpha(alphaLevel)
	if not tonumber(alphaLevel) then
		Swords.backgroundAlpha = 0
	else
		Swords.backgroundAlpha = tonumber(alphaLevel)
	end
	
	Swords.HideSwords()
	Swords.CreateImageFrames()
	Swords.SetVisible()
end

function Swords.SplitArgs(str)
	local result = {}
	local count = 1
	for token in string.gmatch(str, "[^%s]+") do
		result[count] = token
		count = count + 1
	end
	return result
end


--
-- Event Handlers
--

function Swords.ComboChanged(points)
	local playerID = Swords.GetPlayerID()
	if playerID and points[playerID] then
		Swords.swords = points[playerID]
		Swords.SetVisible()
	end
end

function Swords.SetInCombat()
	Swords.inCombat = true
	Swords.SetVisible()
end

function Swords.SetOutCombat()
	Swords.inCombat = false
	Swords.SetVisible()
end

function Swords.Initialise(addon)
	if addon == Swords.name then
		Swords.Create()
		SwordsTimer.AddCallback("SwordsStart", 1, Swords.Start)
	end
end

function Swords.SlashHandler(args)
	local arg = Swords.SplitArgs(args)
	
	if arg[1] == "on" then
		Swords.visible = true
		Swords.SetVisible()
	elseif arg[1] == "off" then
		Swords.visible = false
		Swords.SetVisible()
	elseif arg[1] == "always" then
		Swords.visible = true
		Swords.mode = arg[1]
		Swords.SetVisible()
	elseif arg[1] == "combat" then
		Swords.visible = true
		Swords.mode = arg[1]
		Swords.SetVisible()
	elseif arg[1] == "lock" then
		Swords.DisableDragFrame()
	elseif arg[1] == "unlock" then
		Swords.EnableDragFrame()
	elseif arg[1] == "image" then
		Swords.SetImage(arg[2])
	elseif arg[1] == "offset" then
		Swords.SetImageOffset(arg[2])
	elseif arg[1] == "scale" then
		Swords.SetImageScale(arg[2])
	elseif arg[1] == "background" then
		Swords.SetBackgroundAlpha(arg[2])
	elseif arg[1] == "reset" then
		Swords.DisableDragFrame()
		Swords.x = 500
		Swords.y = 500
		Swords.visible = true
		Swords.mode = "combat"
		Swords.inCombat = false
		Swords.image = "dagger"
		Swords.imageOffset = 50
		Swords.backgroundAlpha = 0
	elseif not arg[1] then
		Swords.SetVisible()
	else
		print("Usage: swords [ on | off | always | combat | lock | unlock | image [ imagename ] | offset [ imageoffset ] | scale [ scaleFactor ] | background [ alpha ] | reset ]")
		return
	end
	
	local message = "Swords is "
	if Swords.visible == true then
		message = message .. "on "
		if Swords.mode == "combat" then
			message = message .. "and will only show in combat"
		else
			message = message .. "and will show both in and out of combat"
		end
		
		if Swords.dragFrame then
			message = message .. ". Unlocked so right click to drag"
		end
		
		message = message .. ". Swords is using image set " .. Swords.image
	else
		message = message .. "off "
	end
	print(message)
end

function Swords.SaveVariables(addon)
	if addon == Swords.name then
		-- now copy saved group to settings so that they can be preserved on logout
		Swords_SavedVariables = {}
		Swords_SavedVariables.version = Swords.version
		Swords_SavedVariables.x = Swords.x
		Swords_SavedVariables.y = Swords.y
		Swords_SavedVariables.visible = Swords.visible
		Swords_SavedVariables.mode = Swords.mode
		Swords_SavedVariables.image = Swords.image
		Swords_SavedVariables.imageOffset = Swords.imageOffset
		Swords_SavedVariables.imageScale = Swords.imageScale
		Swords_SavedVariables.backgroundAlpha = Swords.backgroundAlpha
	end
end

function Swords.GetSavedValue(value, default)
	if value ~= nil then
		return value
	else
		return default
	end
end

function Swords.LoadVariables(addon)
	if addon == Swords.name then
		-- now that variables are loaded and saved positions restored we can create frame
		if Swords_SavedVariables then
			Swords.x = Swords.GetSavedValue(Swords_SavedVariables.x, 500)
			Swords.y = Swords.GetSavedValue(Swords_SavedVariables.y, 500)
			Swords.visible = Swords.GetSavedValue(Swords_SavedVariables.visible, true)
			Swords.mode = Swords.GetSavedValue(Swords_SavedVariables.mode, "combat")
			Swords.image = Swords.GetSavedValue(Swords_SavedVariables.image, "dagger")
			Swords.imageOffset = Swords.GetSavedValue(Swords_SavedVariables.imageOffset, 50)
			Swords.imageScale = Swords.GetSavedValue(Swords_SavedVariables.imageScale, 1)
			Swords.backgroundAlpha = Swords.GetSavedValue(Swords_SavedVariables.backgroundAlpha, 0)

			if Swords.image == "1h_dagger_019" then
				Swords.image = "dagger"
			end
		end
	end	
end

function Swords.MouseDown(dragFrame)
	local mouse = Inspect.Mouse()
	dragFrame.x = dragFrame.frame:GetLeft()
	dragFrame.y = dragFrame.frame:GetTop()
	dragFrame.xOffset = dragFrame.x - mouse.x
	dragFrame.yOffset = dragFrame.y - mouse.y
	dragFrame.dragging = true
end

function Swords.MouseMove(dragFrame, x, y)
	if dragFrame.dragging == true then
		Swords.x = x + dragFrame.xOffset
		Swords.y = y + dragFrame.yOffset
		Swords.parentFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Swords.x, Swords.y)
	end
end

function Swords.MouseUp(dragFrame)
	dragFrame.dragging = false
end

--
-- Register events
--
SwordsTimer.Initialise(Swords.name)
table.insert(Command.Slash.Register(Swords.slashName), { Swords.SlashHandler, Swords.name, "SlashHandler"})
table.insert(Event.Addon.Load.End, { Swords.Initialise, Swords.name, "Initialise" })
table.insert(Event.Addon.SavedVariables.Save.Begin, { Swords.SaveVariables, Swords.name, "SaveVariables" })
table.insert(Event.Addon.SavedVariables.Load.End, { Swords.LoadVariables, Swords.name, "LoadVariables" })
table.insert(Event.Unit.Detail.Combo, { Swords.ComboChanged, Swords.name, "ComboChanged" })
table.insert(Event.System.Secure.Enter, { Swords.SetInCombat, Swords.name, "SetInCombat" })
table.insert(Event.System.Secure.Leave, { Swords.SetOutCombat, Swords.name, "SetInCombat" })
