-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

SwordsTimer = {}
SwordsTimer.name = "SwordsTimer"
SwordsTimer.version = "v0.1"
SwordsTimer.eventsByTime = {}
SwordsTimer.eventsByID = {}
SwordsTimer.eventID = 0
SwordsTimer.currentTime = 0

--
-- Main functions
--

function SwordsTimer.Initialise(addonName)
	table.insert(Event.System.Update.Begin, { SwordsTimer.Refresh, addonName, "SwordsTimer" })
end

function SwordsTimer.Clear()
	SwordsTimer.eventsByTime = {}
	SwordsTimer.eventsByID = {}
end

function SwordsTimer.AddCallback(name, duration, callback, repeatIt)
	if SwordsTimer.currentTime == 0 then
		SwordsTimer.currentTime = SwordsTimer.InternalGetTime()
	end
	
	local eventID = SwordsTimer.InternalGetID()
	SwordsTimer.InternalAdd(eventID, name, duration, callback, repeatIt)
	return eventID
end

function SwordsTimer.RemoveCallback(timerId)
	local thisSwordsTimer = SwordsTimer.eventsByID[timerId]
	if thisSwordsTimer then
		local dueTimeKey = SwordsTimer.GetTimeKey(thisSwordsTimer.dueTime)
		SwordsTimer.eventsByID[timerId] = nil
		SwordsTimer.eventsByTime[dueTimeKey][timerId] = nil
		
		local deleteTable = true
		for i, j in pairs(SwordsTimer.eventsByTime[dueTimeKey]) do
			deleteTable = false
		end
		
		if deleteTable == true then
			SwordsTimer.eventsByTime[dueTimeKey] = nil
		end
	end
end

function SwordsTimer.InternalAdd(id, name, duration, callback, repeatIt)
	local dueTime = SwordsTimer.currentTime + (duration * 10)
	
	local thisSwordsTimer = {}
	thisSwordsTimer.id = id
	thisSwordsTimer.name = name
	thisSwordsTimer.duration = duration
	thisSwordsTimer.callback = callback
	thisSwordsTimer.dueTime = dueTime
	if (repeatIt == true) then
		thisSwordsTimer.repeatSwordsTimer = true
	else
		thisSwordsTimer.repeatSwordsTimer = false
	end
	
	local dueTimeKey = SwordsTimer.GetTimeKey(dueTime)
	if not SwordsTimer.eventsByTime[dueTimeKey] then
		SwordsTimer.eventsByTime[dueTimeKey] = {}
	end

	SwordsTimer.eventsByTime[dueTimeKey][id] = thisSwordsTimer
	SwordsTimer.eventsByID[id] = thisSwordsTimer
end

function SwordsTimer.InternalGetTime()
	return math.floor(Inspect.Time.Frame() * 10)
end

function SwordsTimer.InternalGetID()
	SwordsTimer.eventID = SwordsTimer.eventID + 1
	return "#" .. SwordsTimer.eventID
end

function SwordsTimer.GetTimeKey(thisTime)
	return "@" .. thisTime
end


--
-- Event Handlers
--

function SwordsTimer.Refresh()
	local newTime = SwordsTimer.InternalGetTime()
	
	if newTime ~= SwordsTimer.currentTime then
		local nextSwordsTimerTime = SwordsTimer.currentTime + 1
		for currentTime = nextSwordsTimerTime, newTime do
			SwordsTimer.currentTime = currentTime
			local timeKey = SwordsTimer.GetTimeKey(SwordsTimer.currentTime)
			if SwordsTimer.eventsByTime[timeKey] then
				for id, thisSwordsTimer in pairs(SwordsTimer.eventsByTime[timeKey]) do
					if thisSwordsTimer.repeatSwordsTimer == true then
						SwordsTimer.InternalAdd(thisSwordsTimer.id, thisSwordsTimer.name, thisSwordsTimer.duration, thisSwordsTimer.callback, thisSwordsTimer.repeatSwordsTimer)
					else
						SwordsTimer.eventsByID[thisSwordsTimer.id] = nil
					end

					thisSwordsTimer.callback()
				end

				SwordsTimer.eventsByTime[timeKey] = nil
			end
		end
	end
end

